package com.tutego.jrtf;

/**
 * Represents a RTF header. Different subclasses defines specific headers for
 * <ul>
 * <li>font definitions
 * <li>color tables
 * <li>style sheets (not implemented yet)
 * </ul>
 */
public abstract class RtfHeader
{
  /**
   * Package visible constructor. The user will not instantiate this class.
   */
  RtfHeader() { }
  
  /**
   * Creates a new font definition.
   * @param fontname  Name of the font.
   * @return {@link RtfHeaderFont} to configure the font.
   */
  public static RtfHeaderFont font( String fontname )
  {
    return new RtfHeaderFont( fontname );
  }

  /**
   * Creates a new color definition.
   * From the color values just the last 8 bit are used.
   * @param r Red.
   * @param g Green.
   * @param b Blue.
   * @return {@link RtfHeaderColor} to set the position of the font.
   */
  public static RtfHeaderColor color( int r, int g, int b )
  {
    return new RtfHeaderColor( r & 0xFF, g & 0xFF, b & 0xFF );
  }

//  
//  /**
//   * The default style has to be on position 0.
//   */
//   public static RtfHeaderStyle defaultStyle()
//   {
//    return new RtfHeaderStyle( "", "" );
//   }
//   
//   public static RtfHeaderStyle parStyle( String stylename )
//   {
//    return new RtfHeaderStyle( "\\s", stylename );
//   }
//   
//   public static RtfHeaderStyle charStyle( String stylename )
//   {
//    return new RtfHeaderStyle( "\\cs", stylename );
//   }
}
