package com.tutego.jrtf;

/**
 * Defines properties of the hole row. The properties for a row, e.g. borders,
 * are assigned to every cell.
 */
public abstract class RtfRow extends RtfPara
{
  /*
   * <row>   :=  <tbldef> <cell>+ \row
   * <cell>  :=  <textpar>+ \cell
   */
  
  /** Collects the RTF for the table definitions. */
  StringBuilder tbldef = new StringBuilder();

  // Row Formatting

  //  /**
  //   * AutoFit is on for the row. No auto fit is the default.
  //   * @return {@code this}-object.
  //   */
  //  public RtfRow autoFit()
  //  {
  //    tbldef.append( "\\trautofit1" );
  //    return this;
  //  }

  /**
   * Table direction is right to left.
   * @return {@code this}-object.
   */
  public RtfRow rightToLeft()
  {
    tbldef.append( "\\taprtl" );
    return this;
  }

//  /**
//   * Default bottom cell margin or padding for the row.
//   * @param margin
//   * @param unit
//   * @return
//   */
//  public RtfRow bottomCellMargin( double margin, RtfUnit unit )
//  {
//    tbldef.append( "\\trpaddfb3\\trpaddb" ).append( unit.toTwips( margin ) );
//    return this;
//  }
//
//  /**
//   * Default left cell margin or padding for the row.
//   * @param margin
//   * @param unit
//   * @return
//   */
//  public RtfRow leftCellMargin( double margin, RtfUnit unit )
//  {
//    tbldef.append( "\\trpaddl3\\trpaddl" ).append( unit.toTwips( margin ) );
//    return this;
//  }
//
//  /**
//   * Default right cell margin or padding for the row.
//   * @param margin
//   * @param unit
//   * @return
//   */
//  public RtfRow rightCellMargin( double margin, RtfUnit unit )
//  {
//    tbldef.append( "\\trpaddr3\\trpaddr" ).append( unit.toTwips( margin ) );
//    return this;
//  }
//
//  /**
//   * Default top cell margin or padding for the row.
//   * @param margin
//   * @param unit
//   * @return
//   */
//  public RtfRow topCellMargin( double margin, RtfUnit unit )
//  {
//    tbldef.append( "\\trpaddt3\\trpaddt" ).append( unit.toTwips( margin ) );
//    return this;
//  }

  // Cell Borders
  
  /**
   * Bottom table cell border.
   * @return {@code this}-object.
   */
  public RtfRow bottomCellBorder()
  {
    tbldef.append( "\\clbrdrb\\brdrs" );
    return this;
  }

  /**
   * Top table cell border.
   * @return {@code this}-object.
   */
  public RtfRow topCellBorder()
  {
    tbldef.append( "\\clbrdrt\\brdrs" );
    return this;
  }

  /**
   * Left table cell border.
   * @return {@code this}-object.
   */
  public RtfRow leftCellBorder()
  {
    tbldef.append( "\\clbrdrl\\brdrs" );
    return this;
  }

  /**
   * Right table cell border.
   * @return {@code this}-object.
   */
  public RtfRow rightCellBorder()
  {
    tbldef.append( "\\clbrdrr\\brdrs" );
    return this;
  }

  /**
   * Half the space between the cells of a table row.
   * @param space Space between cells.
   * @param unit Measurement.
   * @return {@code this}-object.
   */
  public RtfRow cellSpace( double space, RtfUnit unit )
  {
    tbldef.append( "\\trgaph" ).append( unit.toTwips( space ) );
    return this;
  }

  /**
   * Defines the cell height.
   * @param height Height of the cell.
   * @param unit Measurement.
   * @return {@code this}-object.
   */
  public RtfRow cellHeight( double height, RtfUnit unit )
  {
    tbldef.append( "\\trrh" ).append( unit.toTwips( height ) );
    return this;
  }  
}
